use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};

use crate::{def_ld, ObjectSlot, ObjectTyped, OneOrMore};
use crate::object::ObjectLinkUnion;

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct Activity<'a> {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#actor")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub actor: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#object")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub object: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#target")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub target: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#result")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub result: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#origin")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub origin: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#instrument")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub instrument: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Activity", Activity<'_>, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct IntransitiveActivity {}

def_ld!(
    "https://www.w3.org/ns/activitystreams#IntransitiveActivity",
    IntransitiveActivity,
    base as Activity
);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityAccept {}

def_ld!("https://www.w3.org/ns/activitystreams#Accept", ActivityAccept, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityTentativeAccept {}

def_ld!(
    "https://www.w3.org/ns/activitystreams#TentativeAccept",
    ActivityTentativeAccept,
    base as ActivityAccept
);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityAdd;

def_ld!("https://www.w3.org/ns/activitystreams#Add", ActivityAdd, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityArrive;

def_ld!("https://www.w3.org/ns/activitystreams#Arrive", ActivityArrive, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityCreate;

def_ld!("https://www.w3.org/ns/activitystreams#Create", ActivityCreate, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityDelete {}

def_ld!("https://www.w3.org/ns/activitystreams#Delete", ActivityDelete, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityFollow {}

def_ld!("https://www.w3.org/ns/activitystreams#Follow", ActivityFollow, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityIgnore {}

def_ld!("https://www.w3.org/ns/activitystreams#Ignore", ActivityIgnore, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityJoin {}

def_ld!("https://www.w3.org/ns/activitystreams#Join", ActivityJoin, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityLeave {}

def_ld!("https://www.w3.org/ns/activitystreams#Leave", ActivityLeave, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityLike {}

def_ld!("https://www.w3.org/ns/activitystreams#Like", ActivityLike, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityOffer {}

def_ld!("https://www.w3.org/ns/activitystreams#Offer", ActivityOffer, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityInvite {}

def_ld!("https://www.w3.org/ns/activitystreams#Invite", ActivityInvite, base as ActivityOffer);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityReject {}

def_ld!("https://www.w3.org/ns/activitystreams#Reject", ActivityReject, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityTentativeReject {}

def_ld!(
    "https://www.w3.org/ns/activitystreams#TentativeReject",
    ActivityTentativeReject,
    base as ActivityReject
);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityRemove {}

def_ld!("https://www.w3.org/ns/activitystreams#Remove", ActivityRemove, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityUndo {}

def_ld!("https://www.w3.org/ns/activitystreams#Undo", ActivityUndo, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityUpdate {}

def_ld!("https://www.w3.org/ns/activitystreams#Update", ActivityUpdate, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityView {}

def_ld!("https://www.w3.org/ns/activitystreams#View", ActivityView, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityListen {}

def_ld!("https://www.w3.org/ns/activitystreams#Listen", ActivityListen, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityRead {}

def_ld!("https://www.w3.org/ns/activitystreams#Read", ActivityRead, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityMove {}

def_ld!("https://www.w3.org/ns/activitystreams#Move", ActivityMove, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityTravel {}

def_ld!("https://www.w3.org/ns/activitystreams#Travel", ActivityTravel, base as IntransitiveActivity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityAnnounce {}

def_ld!("https://www.w3.org/ns/activitystreams#Announce", ActivityAnnounce, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityBlock {}

def_ld!("https://www.w3.org/ns/activitystreams#Block", ActivityBlock, base as ActivityIgnore);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityFlag {}

def_ld!("https://www.w3.org/ns/activitystreams#Flag", ActivityFlag, base as Activity);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityDislike {}

def_ld!("https://www.w3.org/ns/activitystreams#Dislike", ActivityDislike, base as Activity);

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(untagged)]
pub enum ActivityClosedStatus<'a> {
    Date(DateTime<Utc>),
    Bool(bool),
    Other(ObjectSlot<'a, ObjectLinkUnion>),
}

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActivityQuestion<'a> {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#oneOf")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub one_of: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#anyOf")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub any_of: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#closed")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub closed: Option<OneOrMore<ActivityClosedStatus<'a>>>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Question", ActivityQuestion<'_>, base as IntransitiveActivity);
