use serde::{Deserialize, Serialize};
use serde_json::Value;

use crate::{def_ld, ObjectSlot};
use crate::object::{Collection, ObjectLinkUnion, OrderedCollection};

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorActivityPubProps<'a> {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#inbox")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub inbox: Option<ObjectSlot<'a, OrderedCollection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#outbox")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub outbox: Option<ObjectSlot<'a, OrderedCollection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#following")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub following: Option<ObjectSlot<'a, Collection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#followers")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub followers: Option<ObjectSlot<'a, Collection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#liked")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub liked: Option<ObjectSlot<'a, Collection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#shares")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub shares: Option<ObjectSlot<'a, Collection>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#preferredUsername")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub preferred_username: Option<String>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#endpoints")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub endpoints: Option<Value>,
}

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorApplication {}

def_ld!("https://www.w3.org/ns/activitystreams#Application", ActorApplication, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorGroup {}

def_ld!("https://www.w3.org/ns/activitystreams#Group", ActorGroup, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorOrganization {}

def_ld!("https://www.w3.org/ns/activitystreams#Organization", ActorOrganization, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorPerson {}

def_ld!("https://www.w3.org/ns/activitystreams#Person", ActorPerson, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ActorService {}

def_ld!("https://www.w3.org/ns/activitystreams#Service", ActorService, base as Object);
