use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};

use crate::{def_ld, ld_union, ObjectSlot, ObjectTyped, OneOrMore};
use crate::link::Link;
use crate::object::{Object, ObjectLinkUnion};

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectRelationship<'a> {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#object")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub object: Option<OneOrMore<ObjectSlot<'a, ObjectLinkUnion>>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#subject")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub subject: Option<ObjectSlot<'a, ObjectLinkUnion>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#relationship")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub relationship: Option<OneOrMore<ObjectSlot<'a, Object<'a>>>>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Relationship", ObjectRelationship, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectArticle {}

def_ld!("https://www.w3.org/ns/activitystreams#Article", ObjectArticle, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectDocument {}

def_ld!("https://www.w3.org/ns/activitystreams#Document", ObjectDocument, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectAudio {}

def_ld!("https://www.w3.org/ns/activitystreams#Audio", ObjectAudio, base as ObjectDocument);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectImage {}

def_ld!("https://www.w3.org/ns/activitystreams#Image", ObjectImage, base as ObjectDocument);
ld_union!(
    ObjectImageLinkUnion,
    image_props as ObjectImage,
    link_props as Link
);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectVideo {}

def_ld!("https://www.w3.org/ns/activitystreams#Video", ObjectVideo, base as ObjectDocument);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectNote {}

def_ld!("https://www.w3.org/ns/activitystreams#Note", ObjectNote, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectPage {}

def_ld!("https://www.w3.org/ns/activitystreams#Page", ObjectPage, base as ObjectDocument);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectEvent {}

def_ld!("https://www.w3.org/ns/activitystreams#Event", ObjectEvent, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectPlace {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#accuracy")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub accuracy: Option<f64>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#altitude")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub altitude: Option<f64>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#latitude")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub latitude: Option<f64>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#longitude")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub longitude: Option<f64>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#radius")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub radius: Option<f64>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#units")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub units: Option<String>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Place", ObjectPlace, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectProfile<'a> {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#describes")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub describes: Option<ObjectSlot<'a, ObjectProfile<'a>>>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Profile", ObjectProfile, base as Object);

#[derive(Clone, Default, Debug, Serialize, Deserialize)]
pub struct ObjectTombstone {
    #[serde(rename = "https://www.w3.org/ns/activitystreams#formerType")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub former_type: Option<OneOrMore<String>>,

    #[serde(rename = "https://www.w3.org/ns/activitystreams#deleted")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub deleted: Option<DateTime<Utc>>,
}

def_ld!("https://www.w3.org/ns/activitystreams#Tombstone", ObjectTombstone, base as Object);
